<?php

// Internal status flags (for bitwise operations)
/** @constant PROJECT_ANY Project has any flag set. */
define('PROJECT_ANY', 0);

/** @constant PROJECT_ADDED Project has just been added. */
define('PROJECT_ADDED', 1);

/** @constant PROJECT_MODIFIED Project has been modified. */
define('PROJECT_MODIFIED', 2);

/** @constant PROJECT_DELETED Project has been deleted. */
define('PROJECT_DELETED', 4);

/** @constant PROJECT_ANY Project has any flag set. */
define('IMAGE_ANY', 0);

/** @constant PROJECT_ADDED Project has just been added. */
define('IMAGE_ADDED', 1);

/** @constant PROJECT_MODIFIED Project has been modified. */
define('IMAGE_MODIFIED', 2);

/** @constant PROJECT_DELETED Project has been deleted. */
define('IMAGE_DELETED', 4);

/**
 * igal_Driver:: defines an API for implementing storage backends for igal.
 *
 * $Horde: igal/lib/Driver.php,v 1.34 2003/01/06 09:36:22 jan Exp $
 *
 * @author  Jon Parise <jon@horde.org>
 * @version $Revision: 1.34 $
 * @since   igal 0.1
 * @package igal
 */
class Igal_Driver {

    /**
     * Array holding the current project list.  Each array entry is a hash
     * describing a project.  The array is indexed numerically by project ID.
     * @var array $projects
     */
    var $_projects = array();

    /**
     * Array holding the current images list.  Each array entry is a hash
     * describing a images.  The array is indexed numerically by project ID.
     * @var array $images
     */
    var $_images = array();

    /**
     * String containing the current username.  This indicates the owner of
     * the current project list.
     * @var string $user
     */
    var $_user = '';

    /**
     * Attempts to return a concrete igal_Driver instance based on $driver.
     *
     * @param string    $driver     The type of concrete igal_Driver subclass
     *                              to return.  The is based on the storage
     *                              driver ($driver).  The code is dynamically
     *                              included.
     *
     * @param string    $user       The name of the user who owns these projects.
     *
     * @param array     $params     (optional) A hash containing any additional
     *                              configuration or connection parameters a
     *                              subclass might need.
     *
     * @return mixed    The newly created concrete igal_Driver instance, or
     *                  false on an error.
     */
    function &factory($driver, $user, $params = array())
    {
        $driver = basename($driver);
        require_once dirname(__FILE__) . '/Driver/' . $driver . '.php';
        $class = 'igal_Driver_' . $driver;
        if (class_exists($class)) {
            return new $class($user, $params);
        } else {
            return false;
        }
    }

    /**
     * Attempts to return a reference to a concrete igal_Driver instance
     * based on $driver.  It wil only create a new instance if no
     * igal_Driver instance with the same parameters currently exists.
     *
     * This should be used if multiple storage sources are required.
     *
     * This method must be invoked as: $var = &igal_Driver::singleton()
     *
     * @param string    $driver     The type of concrete igal_Driver subclass
     *                              to return.  The is based on the storage
     *                              driver ($driver).  The code is dynamically
     *                              included.
     *
     * @param string    $user       The name of the user who owns these projects.
     *
     * @param array     $params     (optional) A hash containing any additional
     *                              configuration or connection parameters a
     *                              subclass might need.
     *
     * @return mixed    The created concrete igal_Driver instance, or false
     *                  on error.
     */
    function &singleton($driver, $user, $params = array())
    {
        static $instances;

        if (!isset($instances)) {
            $instances = array();
        }

        $signature = serialize(array($driver, $user, $params));
        if (!isset($instances[$signature])) {
            $instances[$signature] = &igal_Driver::factory($driver, $user, $params);
        }

        return $instances[$signature];
    }

    /**
     * Lists projects based on the given criteria.  All projects will be returned
     * by default.
     *
     * @param optional constant $criteria  A set of flags specifying the
     *                                     set of projects to return.
     *
     * @return array    Returns a list of the requested projects.
     */
    function listProjects($criteria = PROJECT_ANY)
    {
        /* Return all of the projects by default. */
        if ($criteria == PROJECT_ANY) {
            return $this->_projects;
        }

        /* Otherwise, return only the set of projects that match $criteria. */
        $return_set = array();
        foreach ($this->_projects as $project_id => $project) {
            if ($this->_projects[$project_id]['flags'] & $criteria) {
                $return_set[$project_id] = $project;
            }
        }

        return $return_set;
    }

    /**
     * Adds a new project to the $projects list.
     *
     * @param string           $name      The name (short) of the project.
     * @param string           $desc      The description (long) of the project.
     * @param optional integer $due       The due date of the project.
     * @param optional integer $priority  The priority of the project.
     * @param optional integer $completed The completion state of the project.
     *
     * @return integer  The numeric ID of the new project.
     */
    function addProject($name, $desc, $category = 0)
    {
        /* Create a new project project. */
        $project = array();
        $project['name'] = $name;
        $project['desc'] = $desc;
        $project['added'] = time();
        $project['category'] = $category;
        $project['flags'] = PROJECT_ADDED;

        /* Add it to the $projects list. */
        $this->_projects[] = $project;

        $keys = array_keys($this->_projects);
	
        return $keys[(count($keys) - 1)];
    }

    /**
     * Modifies an attribute of a project in the $projects list.
     *
     * @param int       $id         The ID of the project to modify.
     * @param string    $attribute  The attribute of the project to modify.
     * @param mixed     $value      The new value for $attribute.
     *
     * @return boolean  True if the project was successfully marked as modified.
     */
    function modifyProject($id, $attribute, $value)
    {
        if (isset($this->_projects[$id]) && isset($this->_projects[$id][String::lower($attribute)])) {
            $this->_projects[$id][String::lower($attribute)] = $value;
            $this->_projects[$id]['flags'] |= PROJECT_MODIFIED;
            return ($this->_projects[$id]['flags'] & PROJECT_MODIFIED);
        }

        return false;
    }

    /**
     * Removes a project from the $projects list.
     *
     * @param int       $id         The ID of the project to delete.
     *
     * @return boolean  True if the project was successfully marked as deleted.
     */
    function deleteProject($id)
    {
        if (array_key_exists($id, $this->_projects)) {
            $this->_projects[$id]['flags'] |= PROJECT_DELETED;
            return ($this->_projects[$id]['flags'] & PROJECT_DELETED);
        }

        return false;
    }

    /**
     * Sets the requested flag to the given value.
     *
     * @param int             $id         The ID of the project project to modify.
     * @param int             $flag       The flag to modify.
     * @param boolean         $state      The new state of the flag.
     * @param optional string $item       Which object requested, image or project
     */
    function setFlag($id, $flag, $state, $item = 'project')
    {
        $item = '_' . $item . 's';
        if (isset($this->$item[$id])) {
            if ($state) {
                $this->$item[$id]['flags'] |= $flag;
            } else {
                $this->$item[$id]['flags'] &= ~$flag;
            }
        }
    }

    /**
     * Gets the requested flag's value.
     *
     * @param int               $id         The ID of the project project to examine.
     * @param int               $flag       The flag whose value to return.
     * @param optional string	$item       Which object requested image or project
     *
     * @return boolean  The state of the requested flag.
     */
    function getFlag($id, $flag, $item = 'project')
    {
        $item = '_' . $item . 's';
        if (isset($this->$item[$id])) {
            return ($this->$item[$id]['flags'] & $flag);
        }

        return null;
    }

    /**
     * Purges those projects that have the "deleted" flag set.
     *
     * @param optional string   $item       Which object requested image or project
     */
    function purgeDeleted($item = 'project')
    {
        if ($item == 'project') {
            $deleted_ids = array_keys($this->listProjects(PROJECT_DELETED));
            foreach ($deleted_ids as $project_id) {
                unset($this->_projects[$project_id]);
            }
        }
        elseif ($item == 'image') {
            $deleted_ids = array_keys($this->listImages(IMAGE_DELETED));
            foreach ($deleted_ids as $image_id) {
                unset($this->_images[$image_id]);
            }
        }
    }

    /**
     * Delete all projects from the list
     *
     * @param optional string   $item       Which object requested image or project
     *
     */
    function deleteAll($item = 'project')
    {
        if ($item == 'project') {
            foreach ($this->_projects as $id => $project) {
                $this->_projects[$id]['flags'] |= PROJECT_DELETED;
            }
        }
        elseif ($item == 'image') {
            foreach ($this->_images as $id => $image) {
                $this->_images[$id]['flags'] |= IMAGE_DELETED;
            }
        }
    }

    /**
     * Lists imagess based on the given criteria.  All imagess will be returned
     * by default.
     *
     * @param optional constant $criteria  A set of flags specifying the
     *                                     set of images to return.
     *
     * @return array    Returns a list of the requested projects.
     */
    function listImages($criteria = IMAGE_ANY)
    {
        /* Return all of the projects by default. */
        if ($criteria == IMAGE_ANY) {
            return $this->_images;
        }

        /* Otherwise, return only the set of projects that match $criteria. */
        $return_set = array();
        foreach ($this->_images as $image_id => $image) {
            if ($this->_images[$image_id]['flags'] & $criteria) {
                $return_set[$image_id] = $image;
            }
        }

        return $return_set;
    }


    /**
     * Adds a new project to the $projects list.
     *
     * @param string           $name      The name (short) of the Image.
     * @param string           $desc      The description (long) of the Image.
     * @param string           $filename  The filename of the Image.
     *
     * @return integer  The numeric ID of the new image.
     */
    function addImage($project_id, $project_default, $desc, $filename)
    {
        /* Create a new project project. */
        $image = array();
        $image['project_id'] = $project_id;
        $image['owner'] = $this->_user;
        $image['file_name'] = $filename;
        $image['desc'] = $desc;
        $image['project_default'] = $project_default;
        $image['flags'] = IMAGE_ADDED;

        /* Add it to the $projects list. */
        $this->_images[] = $image;

        $keys = array_keys($this->_images);

        return $keys[(count($keys) - 1)];
    }

    /**
     * Modifies an attribute of a project in the $images list.
     *
     * @param int       $id         The ID of the image to modify.
     * @param string    $attribute  The attribute of the image to modify.
     * @param mixed     $value      The new value for $attribute.
     *
     * @return boolean  True if the image was successfully marked as modified.
     */
    function modifyImage($id, $attribute, $value)
    {
        if (isset($this->_images[$id]) && isset($this->_images[$id][String::lower($attribute)])) {
            $this->_images[$id][String::lower($attribute)] = $value;
            $this->_images[$id]['flags'] |= IMAGE_MODIFIED;
            return ($this->_images[$id]['flags'] & IMAGE_MODIFIED);
        }

        return false;
    }

    /**
     * Removes a image from the $images list.
     *
     * @param int       $id         The ID of the image to delete.
     *
     * @return boolean  True if the image was successfully marked as deleted.
     */
    function deleteImage($id)
    {
        if (array_key_exists($id, $this->_images)) {
            $this->_images[$id]['flags'] |= IMAGE_DELETED;
            return ($this->_images[$id]['flags'] & IMAGE_DELETED);
        }

        return false;
    }

}
