<?php
/**
 * igall storage implementation for PHP's PEAR database abstraction layer.
 *
 * Required values for $params:<pre>
 *      'phptype'       The database type (e.g. 'pgsql', 'mysql', etc.).
 *      'hostspec'      The hostname of the database server.
 *      'protocol'      The communication protocol ('tcp', 'unix', etc.).
 *      'username'      The username with which to connect to the database.
 *      'password'      The password associated with 'username'.
 *      'database'      The name of the database.
 *      'table'         The name of the projects table in 'database'.</pre>
 *      'charset'       The database's internal charset.
 *
 * Required by some database implementations:
 *      'options'       Additional options to pass to the database.
 *      'tty'           The TTY on which to connect to the database.
 *      'port'          The port on which to connect to the database.
 *
 * The table structure can be created by the scripts/drivers/gal_projects.sql
 * script.
 *
 * $Horde: igal/lib/Driver/sql.php,v 1.36 2003/01/17 22:43:23 chuck Exp $
 *
 * @author  Jon Parise <jon@horde.org>
 * @version $Revision: 1.36 $
 * @since   igal 0.1
 * @package igal
 */
class igal_Driver_sql extends igal_Driver {

    /** Hash containing connection parameters. */
    var $_params = array();

    /** Handle for the current database connection.
        @var object DB $db */
    var $_db;

    /** Boolean indicating whether or not we're connected to the SQL server. */
    var $_connected = false;

    /**
     * Constructs a new SQL storage object.
     *
     * @param string $user      The user who owns these projects.
     * @param array  $params    A hash containing connection parameters.
     */
    function igal_Driver_sql($user, $params = array())
    {
        $this->_user = $user;
        $this->_params = $params;
    }

    /**
     * Disconnect from the SQL server and clean up the connection.
     *
     * @return boolean     true on success, false on failure.
     */
    function _disconnect()
    {
        if ($this->_connected) {
            $this->_connected = false;
            return $this->_db->disconnect();
        }

        return true;
    }

    /**
     * Retrieves the user's projects from the database.
     *
     * @param optional string $item       Which object requested, image or project
     *
     * @return mixed  True on success, PEAR_Error on failure.
     */
    function retrieveProject()
    {
        /* Make sure we have a valid database connection. */
        $this->_connect();

        /* Build the SQL query. */
        $query = sprintf('SELECT * FROM %s WHERE project_owner = %s',
                         $this->_params['table'], $this->_db->quote($this->_user));

        /* Log the query at a DEBUG log level. */
        Horde::logMessage(sprintf('igal_Driver_sql::retrieve(): %s', $query),
                          __FILE__, __LINE__, LOG_DEBUG);

        /* Execute the query. */
        $result = $this->_db->query($query);

        if (isset($result) && !DB::isError($result)) {
            $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
            if (DB::isError($row)) {
                return $row;
            }

            /* Store the retrieved values in a fresh $projects list. */
            $this->_projects = array();
            while ($row && !DB::isError($row)) {
                /* Create a new project based on this row's values. */
                $project = array();
                $project['project_id'] = $row['project_id'];
                $project['projectlist_id'] = $this->_user;
                $project['name'] = String::convertCharset($row['project_name'], $this->_params['charset']);
                $project['desc'] = String::convertCharset($row['project_desc'], $this->_params['charset']);
                $project['category'] = $row['project_category'];
                $project['flags'] = 0;

                /* Add this new project to the $projects list. */
                $this->_projects[$row['project_id']] = $project;

                /* Advance to the new row in the result set. */
                $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
            }

            $result->free();
        } else {
            return $result;
        }

        return true;
    }

    /**
     * Retrieves the user's projects from the database.
     *
     * @param  $project_id      
     *
     * @return mixed  True on success, PEAR_Error on failure.
     */
    function retrieveImage($project_id)
    {
        global $registry;
        /* Make sure we have a valid database connection. */
        $this->_connect();

        /* Build the SQL query. */
        $query = sprintf('SELECT * FROM %s WHERE project_id =  %d AND image_owner = %s',
                         $this->_params['table'], $project_id, $this->_db->quote($this->_user));

        /* Log the query at a DEBUG log level. */
        Horde::logMessage(sprintf('igal_Driver_sql::retrieveImage(): %s', $query),
                          __FILE__, __LINE__, LOG_DEBUG);

        /* Execute the query. */
        $result = $this->_db->query($query);

        if (isset($result) && !DB::isError($result)) {
            $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
            if (DB::isError($row)) {
                return $row;
            }

            /* Store the retrieved values in a fresh $images list. */
            $this->_images = array();
            while ($row && !DB::isError($row)) {
                /* Create a new image based on this row's values. */
                $image = array();
                $image['project_id'] = $row['project_id'];
                $image['image_id'] = $row['image_id'];
                $image['owner'] = $this->_user;
                $image['file_name'] = String::convertCharset($row['image_file_name'], $this->_params['charset']);
                $image['desc'] = String::convertCharset($row['image_desc'], $this->_params['charset']);
                $image['project_default'] = $row['project_default'];
                $image['flags'] = 0;

                /* get the thumbnail name */
                $image_base = explode('.', $row['image_file_name']);
                $image_ext = array_pop($image_base);
                $image_basename = implode('.', $image_base);
                $thumb_name = $image_basename . '-th.' . $image_ext;
                $image['thumb_name'] = $thumb_name;

                /* get the image location */
                $image['location'] = $registry->getParam('webroot') . $this->_params['incoming_dest'] . '/project_'. $image['owner'] . '_' . $project_id;


                /* Add this new project to the $projects list. */
                $this->_images[$row['image_id']] = $image;

                /* Advance to the new row in the result set. */
                $row = $result->fetchRow(DB_FETCHMODE_ASSOC);
            }

            $result->free();
        } else {
            return $result;
        }

        return true;
    }

    /**
     * Stores the user's projects to SQL server.
     *
     * @return mixed  True on success, PEAR_Error on failure.
     */
    function storeProject()
    {
        global $registry;
        /* Build lists of the projects that require pending database operations. */
        $added_projects = $this->listProjects(PROJECT_ADDED);
        $modified_projects = $this->listProjects(PROJECT_MODIFIED);
        $deleted_projects = $this->listProjects(PROJECT_DELETED);

        /* If there are no pending operations, exit successfully now. */
        if ((count($added_projects) == 0) && (count($modified_projects) == 0) &&
            (count($deleted_projects) == 0)) {
            return true;
        }

        /* Make sure we have a valid database connection. */
        $this->_connect();

        /* Perform any pending additions. */
        if (count($added_projects) > 0) {
            foreach ($added_projects as $project_id => $project) {
                $query = sprintf(
                    'INSERT INTO %s (project_owner, project_id, project_name, ' .
                    'project_desc, project_category) ' .
                    'VALUES (%s, %d, %s, %s, %s)',
                    $this->_params['table'],
                    $this->_db->quote($this->_user),
                    $project_id,
                    String::convertCharset($this->_db->quote($project['name']), NLS::getCharset(), $this->_params['charset']),
                    String::convertCharset($this->_db->quote($project['desc']), NLS::getCharset(), $this->_params['charset']),
                    $project['category']);
                    

                    /* Log the query at a DEBUG log level. */
                    Horde::logMessage(sprintf('igal_Driver_sql::store(): %s', $query),
                                      __FILE__, __LINE__, LOG_DEBUG);

                /* Attempt the insertion query. */
                $result = $this->_db->query($query);

                /* Return an error immediately if the query failed. */
                if (DB::isError($result)) {
                    return $result;
                }

                /* Create folder to hold Project Images */
                $path = IGAL_BASE . $this->_params['incoming_dest'] . '/project_' . $this->_user . '_' . $project_id;
                if (!Igal::ForceDirectories($path, 'create')){
                    /* Log the problem at a DEBUG log level. */
                    Horde::logMessage(sprintf('igal_Driver_sql::store(): There was a Problem Creating the Image Directory: %s', $path),
                                      __FILE__, __LINE__, LOG_DEBUG);
                    return PEAR::raiseError(_("There was a Problem Creating the Image Directory"));
                }

                /* Remove the "added" flag from this project. */
                $this->setFlag($project_id, PROJECT_ADDED, false);
            }
        }

        /* Perform any pending modifications. */
        if (count($modified_projects) > 0) {
            foreach ($modified_projects as $project_id => $project) {
                $query  = sprintf('UPDATE %s SET ', $this->_params['table']);
                $query .= sprintf('project_name = %s, ',
                                  String::convertCharset($this->_db->quote($project['name']), NLS::getCharset(), $this->_params['charset']));
                $query .= sprintf('project_desc = %s, ',
                                  String::convertCharset($this->_db->quote($project['desc']), NLS::getCharset(), $this->_params['charset']));
                $query .= sprintf('project_category = %d ', $project['category']);
                $query .= sprintf('WHERE project_owner = %s AND project_id = %d',
                                  $this->_db->quote($this->_user), $project_id);

                /* Log the query at a DEBUG log level. */
                Horde::logMessage(sprintf('igal_Driver_sql::store(): %s', $query),
                                  __FILE__, __LINE__, LOG_DEBUG);

                /* Attempt the update query. */
                $result = $this->_db->query($query);

                /* Return an error immediately if the query failed. */
                if (DB::isError($result)) {
                    return $result;
                }

                /* Remove the "modified" flag from this project. */
                $this->setFlag($project_id, PROJECT_MODIFIED, false);
            }
        }

        /* Perform any pending deletions. */
        if (count($deleted_projects) > 0) {
            $project_ids = array_keys($deleted_projects);
            $where = 'project_id = ' . $project_ids[0];
            if (count($project_ids) > 1) {
                array_shift($project_ids);
                $where .= ' OR project_id = ' . implode(' OR project_id =', $project_ids);
            }

            $query = sprintf('DELETE FROM %s WHERE project_owner = %s AND (%s)',
                             $this->_params['table'],
                             $this->_db->quote($this->_user),
                             $where);

            /* Log the query at a DEBUG log level. */
            Horde::logMessage(sprintf('igal_Driver_sql::store(): %s', $query),
                              __FILE__, __LINE__, LOG_DEBUG);

            /* Attempt the delete query. */
            $result = $this->_db->query($query);

            /* Return an error immediately if the query failed. */
            if (DB::isError($result)) {
                return $result;
            }

            /* Delete folder to hold Project Images */
            $path = IGAL_BASE . $this->_params['incoming_dest'] . '/project_' . $this->_user . '_' . $project_ids[0];
            if (!Igal::ForceDirectories($path, 'delete')){
                /* Log the problem at a DEBUG log level. */
                Horde::logMessage(sprintf('igal_Driver_sql::store(): There was a Problem Deleting the Image Directory: %s', $path),
                                  __FILE__, __LINE__, LOG_DEBUG);
                return PEAR::raiseError(_("There was a Problem Deleting the Image Directory"));
            }

            /* Purge the deleted projects. */
            $this->purgeDeleted();
        }

        return true;
    }

    /**
     * Stores the user's projects to SQL server.
     *
     * @return mixed  True on success, PEAR_Error on failure.
     */
    function storeImage()
    {
        /* Build lists of the projects that require pending database operations. */
        $added_images = $this->listImages(IMAGE_ADDED);
        $modified_images = $this->listImages(IMAGE_MODIFIED);
        $deleted_images = $this->listImages(IMAGE_DELETED);
                                                                                                                                                   
        /* If there are no pending operations, exit successfully now. */
        if ((count($added_images) == 0) && (count($modified_images) == 0) &&
            (count($deleted_images) == 0)) {
            return true;
        }
                                                                                                                                                   
        /* Make sure we have a valid database connection. */
        $this->_connect();
                                                                                                                                                   
        /* Perform any pending additions. */
        if (count($added_images) > 0) {
            foreach ($added_images as $image_id => $image) {
                $query = sprintf(
                    'INSERT INTO %s (project_id, image_id, image_owner, project_default, ' .
                    'image_desc, image_file_name) ' .
                    'VALUES (%s, %d, %s, %s, %s, %s)',
                    $this->_params['table'],
                    $image['project_id'],
                    $image_id,
                    $this->_db->quote($this->_user),
                    $image['project_default'],
                    String::convertCharset($this->_db->quote($image['desc']), NLS::getCharset(), $this->_params['charset']),
                    String::convertCharset($this->_db->quote($image['file_name']), NLS::getCharset(), $this->_params['charset']));

                    /* Log the query at a DEBUG log level. */
                    Horde::logMessage(sprintf('igal_Driver_sql::storeImage(): %s', $query),
                                      __FILE__, __LINE__, LOG_DEBUG);
                                                                                                                                                   
                /* Attempt the insertion query. */
                $result = $this->_db->query($query);
                                                                                                                                                   
                /* Return an error immediately if the query failed. */
                if (DB::isError($result)) {
                    return $result;
                }

                /* Remove the "added" flag from this project. */
                $this->setFlag($image_id, IMAGE_ADDED, false, 'image');
            }
        }
                                                                                                                                                   
        /* Perform any pending modifications. */
        if (count($modified_images) > 0) {
            foreach ($modified_images as $image_id => $image) {
                $query  = sprintf('UPDATE %s SET ', $this->_params['table']);
                $query .= sprintf('image_desc = %s, ',
                                  String::convertCharset($this->_db->quote($image['desc']), NLS::getCharset(), $this->_params['charset']));
                $query .= sprintf('project_default = %d ', $image['project_default']);
                $query .= sprintf('WHERE image_owner = %s AND image_id = %d',
                                  $this->_db->quote($this->_user), $image_id);
                                                                                                                                                   
                /* Log the query at a DEBUG log level. */
                Horde::logMessage(sprintf('igal_Driver_sql::storeImage(): %s', $query),
                                  __FILE__, __LINE__, LOG_DEBUG);
                                                                                                                                                   
                /* Attempt the update query. */
                $result = $this->_db->query($query);
                                                                                                                                                   
                /* Return an error immediately if the query failed. */
                if (DB::isError($result)) {
                    return $result;
                }
                                                                                                                                                   
                /* Remove the "modified" flag from this project. */
                $this->setFlag($image_id, IMAGE_MODIFIED, false, 'image');
            }
        }
                                                                                                                                                   
        /* Perform any pending deletions. */
        if (count($deleted_images) > 0) {
            $image_ids = array_keys($deleted_images);
            $where = 'image_id = ' . $image_ids[0];
            if (count($image_ids) > 1) {
                array_shift($image_ids);
                $where .= ' OR image_id = ' . implode(' OR image_id =', $image_ids);
            }

            $query = sprintf('DELETE FROM %s WHERE image_owner = %s AND (%s)',
                             $this->_params['table'],
                             $this->_db->quote($this->_user),
                             $where);
                                                                                                                                                   
            /* Log the query at a DEBUG log level. */
            Horde::logMessage(sprintf('igal_Driver_sql::storeImage(): %s', $query),
                              __FILE__, __LINE__, LOG_DEBUG);
                                                                                                                                                   
            /* Attempt the delete query. */
            $result = $this->_db->query($query);
                                                                                                                                                   
            /* Return an error immediately if the query failed. */
            if (DB::isError($result)) {
                return $result;
            }

            foreach($deleted_images as $id => $image) {
                $file = IGAL_BASE . $this->_params['incoming_dest'] . '/project_' . $image['owner'] . '_' . 
                    $image['project_id'] . '/' . $image['file_name'];
                $thumb =  IGAL_BASE . $this->_params['incoming_dest'] . '/project_' . $image['owner'] . '_' . 
                    $image['project_id'] . '/' . $image['thumb_name'];
                if (!unlink($file) || !unlink($thumb)) {
                    return PEAR::raiseError(_("There was a Problem Deleteing the Image"));
                }
            }

            /* Purge the deleted projects. */
            $this->purgeDeleted();
        }
                                                                                                                                                   
        return true;
    }


    /**
     * Attempts to open a persistent connection to the SQL server.
     *
     * @return boolean    True on success; exits (Horde::fatal()) on error.
     */
    function _connect()
    {
        if (!$this->_connected) {
            if (!is_array($this->_params)) {
                Horde::fatal(PEAR::raiseError(_("No configuration information specified for SQL Projects.")), __FILE__, __LINE__);
            }
            if (!isset($this->_params['phptype'])) {
                Horde::fatal(PEAR::raiseError(_("Required 'phptype' not specified in projects configuration.")), __FILE__, __LINE__);
            }
            if (!isset($this->_params['hostspec'])) {
                Horde::fatal(PEAR::raiseError(_("Required 'hostspec' not specified in projects configuration.")), __FILE__, __LINE__);
            }
            if (!isset($this->_params['username'])) {
                Horde::fatal(PEAR::raiseError(_("Required 'username' not specified in projects configuration.")), __FILE__, __LINE__);
            }
            if (!isset($this->_params['password'])) {
                Horde::fatal(PEAR::raiseError(_("Required 'password' not specified in projects configuration.")), __FILE__, __LINE__);
            }
            if (!isset($this->_params['database'])) {
                Horde::fatal(PEAR::raiseError(_("Required 'database' not specified in projects configuration.")), __FILE__, __LINE__);
            }
            if (!isset($this->_params['charset'])) {
                Horde::fatal(PEAR::raiseError(_("Required 'charset' not specified in projects configuration.")), __FILE__, __LINE__);
            }
            if (!isset($this->_params['table'])) {
                Horde::fatal(PEAR::raiseError(_("Required 'table' not specified in projects configuration.")), __FILE__, __LINE__);
            }

            /* Connect to the SQL server using the supplied parameters. */
            require_once 'DB.php';
            $this->_db = &DB::connect($this->_params, true);
            if (DB::isError($this->_db)) {
                Horde::fatal($this->_db, __FILE__, __LINE__);
            }

            /* Enable the "portability" option. */
            $this->_db->setOption('optimize', 'portability');

            $this->_connected = true;
        }

        return true;
    }

}
