<?php

require_once IGAL_BASE . '/lib/constants.php';

/**
 * Array for mapping the completed states to their string representations.
 * @var array $completion_map
 */
$completion_map = array(
    0 => _("No"),
    1 => _("Yes"));

/**
 * IGAL Base Class.
 *
 * $Horde: igal/lib/Igal.php,v 1.68 2003/02/06 18:32:22 chuck Exp $
 *
 * @author Jon Parise <jon@horde.org>
 * @version $Revision: 1.68 $
 * @package igal
 */
class Igal {

    /**
     * Retrieves the current user's project list from storage.
     * This function will also sort the resulting list, if requested.
     *
     * @param object Igal_Driver  $storage   The current storage object.
     * @param constant $sortby   (optional) The field by which to sort.
     *                           (IGAL_SORT_NAME IGAL_SORT_DESC, IGAL_SORT_CATEGORY)
     * @param constant $sortdir  (optional) The direction by which to sort.
     *                           (IGAL_SORT_ASC, IGAL_SORT_DESC)
     *
     * @return array        Returns a list of the requested projects.
     *
     * @see igal_Driver::listProjects()
     */
    function listProjects($sortby = IGAL_SORT_NAME,
                       $sortdir = IGAL_SORT_ASCEND)
    {
        global $prefs, $conf;
        $projects = array();

        /* Sorting criteria for the project list. */
        $sort_functions = array(
            IGAL_SORT_NAME => 'ByName',
            IGAL_SORT_CATEGORY => 'ByCategory',
            IGAL_SORT_DESC => 'ByDesc',
            IGAL_SORT_OWNER => 'ByOwner'
        );

        foreach ($GLOBALS['display_projectlists'] as $projectlist) {
            /* Create a Igal storage instance. */
            $storage = &Igal_Driver::singleton($conf['projects']['driver'], $projectlist,
                                              $conf['projects']['params']);
            $storage->retrieveProject();

            /* Retrieve the project list for storage. */
            $newprojects = $storage->listProjects();

            $projects = array_merge($projects, $newprojects);
        }

        // Sort the array if we have a sort function defined for this
        // field.
        if (array_key_exists($sortby, $sort_functions)) {
            $prefix = ($sortdir == IGAL_SORT_DESCEND) ? '_rsort' : '_sort';
            uasort($projects, array('Igal', $prefix . $sort_functions[$sortby]));
        }

        return $projects;
    }

    /**
     * Retrieves the current user's project list from storage.
     *
     * @param $project_id        What Project to get the images for.
     *
     * @return array        Returns a list of the requested images.
     *
     * @see igal_Driver::listProjects()
     */
    function listImages($project_id)
    {
        global $prefs, $conf;
        $images = array();

        /* Create a Igal storage instance. */
        $storage = &Igal_Driver::singleton($conf['images']['driver'], Auth::getAuth(),
                                           $conf['images']['params']);
        $storage->retrieveImage($project_id);

        /* Retrieve the project list for storage. */
        $newimages = $storage->listImages();

        $images = $newimages;
        return $images;
    }


    function getProject($projectlist, $project)
    {

        global $conf;

        $storage = &Igal_Driver::singleton($conf['projects']['driver'], $projectlist,
                                          $conf['projects']['params']);
        $storage->retrieveProject();
        $allprojects = $storage->listProjects();

        if (array_key_exists($project, $allprojects)) {
            return $allprojects[$project];
        }

        return false;
    }

    function getImage($projectlist, $project, $image)
    {
                                                                                                                                                    
        global $conf;
                                                                                                                                                    
        $storage = &Igal_Driver::singleton($conf['images']['driver'], $projectlist,
                                           $conf['images']['params']);
        $storage->retrieveImage($project);
        $allimages = $storage->listImages();
                                                                                                                                                    
        if (array_key_exists($image, $allimages)) {
            return $allimages[$image];
        }
                                                                                                                                                    
        return false;
    }


    /**
     * List a user's categories
     *
     * @return array A list of categories.
     */
    function listCategories($projectlist = null)
    {
        global $prefs;

        static $catString, $categories;

        $cur = $GLOBALS['igal_shares']->getPrefByShare('project_categories', $projectlist);
        if (is_null($catString) || $catString != $cur) {
            $categories = array(0 => _("Unfiled"));

            $catString = $cur;
            if (empty($catString)) {
                return $categories;
            }

            $cats = explode('|', $catString);
            foreach ($cats as $cat) {
                list($key, $val) = explode(':', $cat);
                $categories[$key] = $val;
            }
        }

        asort($categories);
        return $categories;
    }

    /**
     * List all project lists a user has access to.
     *
     * @param optional boolean $owneronly  Only return projectlists that this
     *                                     user owns? Defaults to false.
     * @param optional integer $permission The permission to filter notepads by.
     *
     * @return array  The project lists.
     */
    function listProjectlists($owneronly = false, $permission = _PERMS_SHOW)
    {
        $projectlists = $GLOBALS['igal_shares']->listShares(Auth::getAuth(), $permission, $owneronly);
        uasort($projectlists, array('Horde_Share', '_sortShares'));
        return $projectlists;
    }

    /**
     * Add a new category
     *
     * @param string  $name     The name of the category to add.
     *
     * @return integer          A valid category id, 0 on failure or
     *                          the new category's id on success.
     */
    function addCategory($name)
    {
        global $prefs;

        if ($prefs->isLocked('project_categories') || empty($name)) {
            return 0;
        }

        $categories = Igal::listCategories();
        if (in_array($name, $categories)) {
            return 0;
        }

        $categories[] = $name;
        unset($categories[0]);

        $cats = array();
        $key = 0;
        foreach ($categories as $key => $cat) {
            $cat = array($key, $cat);
            $cats[] = implode(':', $cat);
        }

        $catString = implode('|', $cats);
        $prefs->setValue('project_categories', $catString);

        return $key;
    }

    /**
     * Delete a category
     *
     * @param integer   $categoryID The id of the category to remove.
     *
     * @return boolean              True on success, false on failure.
     */
    function deleteCategory($categoryID)
    {
        global $prefs;
        $categories = Igal::listCategories();

        if ($prefs->isLocked('project_categories') || !array_key_exists($categoryID, $categories)) {
            return false;
        }

        /* The first category ("Unfiled") cannot be deleted. */
        unset($categories[0]);

        /* Delete the specified category. */
        unset($categories[$categoryID]);

        $cats = array();
        foreach ($categories as $key => $cat) {
            $cat = array($key, $cat);
            $cats[] = implode(':', $cat);
        }

        $catString = implode('|', $cats);
        $prefs->setValue('project_categories', $catString);

        return true;
    }


    /**
     * Rename a category
     *
     * @param integer   $categoryID The id of the category to remove.
     * @param string    $name       The new name of the category.
     *
     * @return boolean              True on success, false on failure.
     */
    function renameCategory($categoryID, $name)
    {
        global $prefs;
        $categories = Igal::listCategories();

        if ($prefs->isLocked('project_categories') || empty($name) ||
                        !array_key_exists($categoryID, $categories) ||
                        in_array($name, $categories)) {
            return false;
        }

        /* The first category ("Unfiled") cannot be renamed. */
        unset($categories[0]);

        /* Assign the new name to the specified category. */
        $categories[$categoryID] = $name;

        $cats = array();
        foreach ($categories as $key => $cat) {
            $cat = array($key, $cat);
            $cats[] = implode(':', $cat);
        }

        $catString = implode('|', $cats);
        $prefs->setValue('project_categories', $catString);

        return true;
    }

    /**
     * Builds the HTML for a project category widget.
     *
     * @param string  $name       The name of the widget.
     * @param integer $selected   (optional) The default category.
     * @param boolean $newheader  (optional) Include a new new category option.
     *
     * @return string       The HTML <select> widget.
     */
    function buildCategoryWidget($name, $selected = false, $newheader = false)
    {
        $html = "<select name=\"$name\">";

        if ($newheader) {
            $html .= '<option value="*new*">' . _("New Category") . "</option>\n";
            $html .= '<option value="">----</option>' . "\n";
        }

        foreach (Igal::listCategories() as $id => $name) {
            $html .= "<option value=\"$id\"";
            $html .= ($id == $selected && $selected !== false) ? ' selected="selected">' : '>';
            $html .= $name . '</option>';
        }
        $html .= '</select>';

        return $html;
    }

    /**
     * Formats the given Unix-style date string.
     *
     * @param string $unixdate     The Unix-style date value to format.
     *
     * @return string       The formatted due date string.
     */
    function formatDate($unixdate = '')
    {
        global $conf;

        if (empty($unixdate)) {
            return '';
        }

        $s  = strftime($conf['view']['date_format'], $unixdate);
        $s .= ' ' . _("at") . ' ';
        $s .= strftime($conf['view']['time_format'], $unixdate);

        if ($unixdate < time()) {
            $s = '<span class="overdue">' . $s . '</span>';
        }

        return $s;
    }

    /**
     * Returns the string matching the given category ID.
     *
     * @param integer $categoryID     The category ID to look up.
     * @param string  $projectlist       The projectlist the category belongs to.
     *
     * @return string       The formatted category string.
     */
    function formatCategory($categoryID = 0, $projectlist = null)
    {
        $categories = Igal::listCategories($projectlist);
        return isset($categories[$categoryID]) ?
            $categories[$categoryID] :
            $categories[0];
    }

    /**
     * Comparison function for sorting projects by name.
     *
     * @param array $a  Project one.
     * @param array $b  Project two.
     *
     * @return integer  1 if project one is greater, -1 if project two is greater; 0 if they are equal.
     */
    function _sortByName($a, $b)
    {
        return strcasecmp($a['name'], $b['name']);
    }

    /**
     * Comparison function for reverse sorting projects by name.
     *
     * @param array $a  Project one.
     * @param array $b  Project two.
     *
     * @return integer  -1 if project one is greater, 1 if project two is greater; 0 if they are equal.
     */
    function _rsortByName($a, $b)
    {
        return strcasecmp($b['name'], $a['name']);
    }

    /**
     * Comparison function for sorting projects by category.
     *
     * @param array $a  Project one.
     * @param array $b  Project two.
     *
     * @return integer  1 if project one is greater, -1 if project two is greater; 0 if they are equal.
     */
    function _sortByCategory($a, $b)
    {
        return strcasecmp(Igal::formatCategory($b['category'], $b['projectlist_id']),
                          Igal::formatCategory($a['category'], $a['projectlist_id']));
    }

    /**
     * Comparison function for reverse sorting projects by category.
     *
     * @param array $a  Project one.
     * @param array $b  Project two.
     *
     * @return integer  -1 if project one is greater, 1 if project two is greater; 0 if they are equal.
     */
    function _rsortByCategory($a, $b)
    {
        return strcasecmp(Igal::formatCategory($b['category'], $b['projectlist_id']),
                          Igal::formatCategory($a['category'], $a['projectlist_id']));
    }

    /**
     * Comparison function for sorting projects by Description.
     *
     * @param array $a  Project one.
     * @param array $b  Project two.
     *
     * @return integer  1 if project one is greater, -1 if project two is greater; 0 if they are equal.
     */
    function _sortByDesc($a, $b)
    {
        if ($a['desc'] == $b['desc']) return 0;
        return ($a['desc'] > $b['desc']) ? -1 : 1;
    }

    /**
     * Comparison function for reverse sorting projects by due date.
     *
     * @param array $a  Project one.
     * @param array $b  Project two.
     *
     * @return integer  -1 if project one is greater, 1 if project two is greater; 0 if they are equal.
     */
    function _rsortByDesc($a, $b)
    {
        if ($a['desc'] == $b['desc']) return 0;
        return ($a['desc'] < $b['desc']) ? -1 : 1;
    }

    /**
     * Comparison function for sorting projects by owner.
     *
     * @param array $a  Project one.
     * @param array $b  Project two.
     *
     * @return integer  1 if project one is greater, -1 if project two is greater; 0 if they are equal.
     */
    function _sortByOwner($a, $b)
    {
        $ashare = $GLOBALS['igal_shares']->getShare($a['projectlist_id']);
        $bshare = $GLOBALS['igal_shares']->getShare($b['projectlist_id']);

        $aowner = $a['projectlist_id'];
        $bowner = $b['projectlist_id'];

        if (!is_a($ashare, 'PEAR_Error') && $aowner != $ashare->getOwner()) {
            $aowner = $ashare->getShareName();
        }
        if (!is_a($bshare, 'PEAR_Error') && $bowner != $bshare->getOwner()) {
            $bowner = $bshare->getShareName();
        }

        return strcasecmp($aowner, $bowner);
    }

    /**
     * Comparison function for reverse sorting projects by owner.
     *
     * @param array $a  Project one.
     * @param array $b  Project two.
     *
     * @return integer  -1 if project one is greater, 1 if project two is greater; 0 if they are equal.
     */
    function _rsortByOwner($a, $b)
    {
        $ashare = $GLOBALS['igal_shares']->getShare($a['projectlist_id']);
        $bshare = $GLOBALS['igal_shares']->getShare($b['projectlist_id']);

        $aowner = $a['projectlist_id'];
        $bowner = $b['projectlist_id'];

        if (!is_a($ashare, 'PEAR_Error') && $aowner != $ashare->getOwner()) {
            $aowner = $ashare->getShareName();
        }
        if (!is_a($bshare, 'PEAR_Error') && $bowner != $bshare->getOwner()) {
            $bowner = $bshare->getShareName();
        }

        return strcasecmp($bowner, $aowner);
    }

    function get_winsize( $width, $height )
    {

        /* Convert width */

        if ($width >= 12) {
            $widthfeet = round($width / 12);

            $inches = ($width - ($widthfeet * 12));
            if ($widthfeet <= 2 && $inches > 0) {
                $widthinches = 6;
            } else { $widthinches = 0; }

        }

        if ($height >= 12) {
            $heightfeet = round($height / 12);

            $inches = ( $height - ($heightfeet * 12 ));
            if ($heightfeet <= 2 && $inches > 0) {
                $heightinches = 6;
            } else { $heightinches = 0; }

        }
        $tmp = ($widthfeet * 1000) + ($widthinches * 100) + ($heightfeet * 10) + ($heightinches * 1);
        return $tmp;
    }

    /* ResizeImg ([image to be resized], [new image size], [thumb size], [destination path for images]) */
    function resizeImage( $image, $newWidth, $thumbWidth, $destfilename )
    {
        // specify your file details
        $current_file = $image;
        $max_width = $newWidth;

        // get the current info on the file
        $current_size = getimagesize($current_file);
        $current_img_width = $current_size[0];
        $current_img_height = $current_size[1];
        $image_base = explode('.', $destfilename);

        #print_r($image_base);
        // this part gets the new thumbnail name
        $image_ext = array_pop($image_base);
        $image_basename = implode('.', $image_base);
        $main_name = $destfilename;

        // determine if the Main image actually needs to be resized
        // and if it does, get the new height for it
        if ($current_img_width > $max_width) {
            $too_big_diff_ratio = $current_img_width/$max_width;
            $new_img_width = $max_width;
            $new_img_height = round($current_img_height/$too_big_diff_ratio);

            // presto chango alacazam
            $make_magick = system("convert -geometry '$new_img_width x $new_img_height' $current_file $main_name", $retval);
            if ($retval) { return 2; }
        }

        // specify your file details
        $current_file = $image;
        $max_width = $thumbWidth;

        // get the current info on the file
        $current_size = getimagesize($current_file);
        $current_img_width = $current_size[0];
        $current_img_height = $current_size[1];
        $image_base = explode(".", $destfilename);

        // this part gets the new thumbnail name
        $image_ext = array_pop($image_base);
        $image_basename = implode('.', $image_base);
        $thumb_name = $image_basename . '-th.' . $image_ext;

        // determine if the image actually needs to be resized
        // and if it does, get the new height for it
        if ($current_img_width > $max_width) {
            $too_big_diff_ratio = $current_img_width/$max_width;
            $new_img_width = $max_width;
            $new_img_height = round($current_img_height/$too_big_diff_ratio);

            // presto chango alacazam
            $make_magick = system("convert -geometry '$new_img_width x $new_img_height' $current_file $thumb_name",  $retval);
            if (($retval)) { return 3; }
            else {  return 1; }

        } else { return 4; }
    }

    function moveImageFile($original_file, $dest_file)
    {
        if(!move_uploaded_file($original_file, $dest_file)) {
            /* Log the Problem at a DEBUG log level. */
            Horde::logMessage(sprintf('IGAL: There was a problem moving the image file from %s to %s', $original_file, $dest_file),
                              __FILE__, __LINE__, LOG_DEBUG);
           return PEAR::raiseError(_("There was a Problem moving the Image File"));
        }
        return true;
    }

    /**
     * function to create/delete folders.
     *
     * @param string     $path  path to create/delete.
     * @param string     $actin create or delete.
     *
     * @return bool  True on Success False on Failure.
     */
    function ForceDirectories($path, $action = 'create')
    {
        if ($action == 'create') {
            if (strlen($path) == 0) {
                return false;
            }
            elseif(is_dir($path)) {
                return false; 
            }
            if(!mkdir($path, 0775)) {
                return false;
            }
            else { return true; }
        }
        elseif($action =='delete') {
            if (is_dir($path)) {
                $handle = opendir($path);
                    while($filename = readdir($handle)) {
                        if ($filename != "." && $filename != "..") {
                            unlink($path . "/" . $filename);
                        }
                    }
                closedir($handle); 
                if (!rmdir($path)) { 
                    return false;
                }
                else { return true; }
            }
            else {
                return false;
            }
        }
    }

    function hasDefaultImage($project_id)
    {
        $images =  Igal::listImages($project_id);
        foreach($images as $id => $image) {
            if ($image['project_default'] ==  1) {
                return true;
            }
        }
        return false;
    }


}
